import * as React from "react";
import { IFormBinder, IDataBinder, IFormBinderInjector } from "./formCore";
import { IdEntity, NumericIdEntity, IOneWayValueConverter, IValueConverter, IInputValueConverter, INumericOptions } from "./formValueConverters";
/**
The (abstract) FormBinder, derive from this calss to create your own custom FormBinder
TComponentProps: The type of the props required on the component being placed inside the form
TDataPropValue: The type of the data property being bound
TComponentPropValue: The type of the target property being bound on the component
*/
export declare abstract class FormBinderBase<TComponentProps, TDataPropValue, TComponentPropValue> implements IFormBinder<TComponentProps, any> {
    /** The name of the data property being bound  */
    protected dataName: string;
    /** The name of the component property being bound  */
    protected propertySet: string;
    /** The converter required to convert 'dataName' to 'propertySet' (TDataPropValue to TComponentPropValue) */
    private valueConverter;
    /** The name of the property being read from the change event currentTarget */
    protected propertyGet: string;
    constructor(
        /** The name of the data property being bound  */
        dataName: string, 
        /** The name of the component property being bound  */
        propertySet: string, 
        /** The converter required to convert 'dataName' to 'propertySet' (TDataPropValue to TComponentPropValue) */
        valueConverter?: IValueConverter<TDataPropValue, TComponentPropValue>, 
        /** The name of the property being read from the change event currentTarget */
        propertyGet?: string);
    /** Sets the React elements properties required in binding */
    setElementProperty(props: TComponentProps, dataBinder: IDataBinder<any>): void;
    /** handle the change event, to modify the dataBinder (safely via this.onChanged(...)), then notifyChanged */
    abstract handleValueChanged(props: TComponentProps, dataBinder: IDataBinder<any>, notifyChanged: () => void): void;
    /** convert the value from element to data, set the dataBinder, and notify if changed */
    protected onChanged(dataBinder: IDataBinder<any>, newValue: TComponentPropValue, notifyChanged: () => void): void;
    /** convert from data value to element property value */
    protected convert(data: TDataPropValue): any;
    /** convert from element property value to data value */
    protected convertBack(value: TComponentPropValue): any;
}
export interface IValueChangeBinderProps<T> {
    value: T;
    onValueChange: (value: T) => void;
}
/** A custom FormBinder that sets 'value' (of type T) and 'onValueChange: (value: T) => void' properties */
export declare class ValueChangeBinder<T> extends FormBinderBase<IValueChangeBinderProps<T>, T, T> {
    handleValueChanged(props: IValueChangeBinderProps<T>, dataBinder: IDataBinder<any>, notifyChanged: () => void): void;
}
/** An input FormBinder that sets native 'value' and 'onChange: (e) => void' properties */
export declare class InputFormBinder<TDataPropValue, TComponentPropValue> extends FormBinderBase<React.DOMAttributes, TDataPropValue, TComponentPropValue> {
    handleValueChanged(props: React.DOMAttributes, dataBinder: IDataBinder<any>, notifyChanged: () => void): void;
}
/** A radio input FormBinder */
export declare class RadioFormBinder<TDataPropValue, TComponentPropValue> extends InputFormBinder<TDataPropValue, TComponentPropValue> {
    setElementProperty(props: React.DOMAttributes, dataBinder: IDataBinder<any>): void;
}
/** Form Binder helpers */
export declare class FormBinder {
    static custom<P>(formBinder: IFormBinder<P, any>): IFormBinderInjector<P>;
    /** hidden */
    static hidden<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes>;
    /** password */
    static password<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes>;
    /** text */
    static text<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes>;
    /** text 'email' */
    static textEmail<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes>;
    private static defaultInputFormBinder<TDataPropValue, TTo>(dataName, type, valueConverter?, propertySet?);
    /** range */
    static range(dataName: string, options?: INumericOptions): IFormBinderInjector<React.DOMAttributes>;
    /** uncontroller text input */
    static defaultText<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes>;
    /** numeric text */
    static textNumeric(dataName: string, options?: INumericOptions): IFormBinderInjector<React.DOMAttributes>;
    /** generic select */
    static selectInject<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes>;
    /** select string */
    static select(dataName: string): IFormBinderInjector<React.DOMAttributes>;
    /** select number */
    static selectNumeric(dataName: string): IFormBinderInjector<React.DOMAttributes>;
    /** select string id */
    static selectId(dataName: string, convertBack: IOneWayValueConverter<string, IdEntity>): any;
    static selectId(dataName: string, items: IdEntity[]): any;
    /** select number id */
    static selectNumericId(dataName: string, convertBack: IOneWayValueConverter<string, NumericIdEntity>): any;
    static selectNumericId(dataName: string, items: NumericIdEntity[]): any;
    /** generic checkbox */
    static checkboxInject<TDataPropValue>(/** helooo */ dataName: string, valueConverter?: IValueConverter<TDataPropValue, boolean>): IFormBinderInjector<React.DOMAttributes>;
    /** checkbox binding to boolean */
    static checkbox(dataName: string): IFormBinderInjector<React.DOMAttributes>;
    /** checkbox binding to boolean derived from trueValue and falseValue equality testing */
    static checkboxConvert<TDataPropValue>(dataName: string, trueValue: TDataPropValue, falseValue: TDataPropValue): IFormBinderInjector<React.DOMAttributes>;
    /** generic input 'radio' */
    static radioInject<TDataPropValue>(dataName: string, value: string, valueConverter: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes>;
    /** input 'radio' string value */
    static radio(dataName: string, value: string): IFormBinderInjector<React.DOMAttributes>;
    /** input 'radio' numeric value */
    static radioNumeric(dataName: string, value: number): IFormBinderInjector<React.DOMAttributes>;
    /** input 'radio' item id string value */
    static radioId(dataName: string, value: IdEntity): IFormBinderInjector<React.DOMAttributes>;
    /** input 'radio' item id string value */
    static radioIdEntity(dataName: string, value: IdEntity): IFormBinderInjector<React.DOMAttributes>;
    /** input 'radio' item id numeric value */
    static radioNumericId(dataName: string, value: NumericIdEntity): IFormBinderInjector<React.DOMAttributes>;
    /** input 'radio' item id numeric value */
    static radioNumericIdEntity(dataName: string, value: NumericIdEntity): IFormBinderInjector<React.DOMAttributes>;
}
